unit NoBordersU;

interface

uses
  shades,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, Mask, Buttons;

type
  TfExperiment = class(TForm)
    Label1: TLabel;
    BitBtn1: TBitBtn;
    Memo1: TMemo;
    buBitmap: TButton;
    buShade: TButton;
    Edit1: TEdit;
    ComboBox1: TComboBox;
    procedure BitBtn1Click(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure buBitmapClick(Sender: TObject);
    procedure buShadeClick(Sender: TObject);
  private
    bm : tbitmap;
    FWallPaper : Twallpaper;
    procedure Paint;   override;
    procedure SetUp3d;
    procedure MouseDown(Button: TMouseButton;Shift: TShiftState; X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer);  override;
    procedure Resize;             override;
    procedure Activate;           override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy;                    override;
  published
    property Wallpaper : Twallpaper read Fwallpaper write FWallpaper;
  end;

var
  fExperiment: TfExperiment;
  bitmapnumber : integer;        // just for illustrating alternatives
  shadenumber  : integer;        // just for illustrating alternatives

const
  BORDERMARGIN = 10;


implementation

{$R *.DFM}


procedure tfExperiment.SetUp3d;
begin

  if assigned(bm) then begin
    bm.free;
    bm := nil;
    application.processmessages;
  end;
  refresh;
  application.processmessages;

  if bitmapnumber < 1 then bitmapnumber := 8;
  if shadenumber  < 0 then shadenumber := 0;

  PictureShadeForm(self,shadestore[shadenumber],'BG0'+format('%2.2d',[bitmapnumber]));
  sinkcontrols(self);
  bm := formbitmap(self);

end;


procedure TfExperiment.BitBtn1Click(Sender: TObject);
begin
  close;
end;


destructor TfExperiment.Destroy;
begin
  bm.free;
  inherited destroy;
end;

procedure TfExperiment.Paint;
begin
  inherited paint;
  if assigned(bm) then canvas.draw(0,0,bm);
end;


(************************************************************
*************************************************************
*************************************************************
*****                                                   *****
*****                                                   *****
*****   USE THE NOBO UNIT AVAILABLE FROM                *****
*****   WWW.EMINENT.DEMON.CO.UK/DELPHI.HTM              *****
*****   IN REAL CODE TO DO THE BORDERLESS               *****
*****   MOVE AND RESIZE  -  SIMPLER AND                 *****
*****   EASIER TO IMPLEMENT                             *****
*****                                                   *****
*****                                                   *****
*************************************************************
*************************************************************
************************************************************)


procedure TfExperiment.MouseMove(Shift: TShiftState; X, Y: Integer);
{ The purpose of this is to indicate to the user that they can resize
  the invisible border by twiddling the cursor to the appropriate shape

  We'll use this shape if we later get a mouse-down do initiate the
  appropriate border resize
}
var
  c : tcursor;
  lm,rm,tm,bm : boolean;
begin
  inherited MouseMove(Shift,X, Y);


  // are we in any BORDERMARGINin areas?
  lm := (abs(x)<BORDERMARGIN);
  tm := (abs(y)<BORDERMARGIN);
  rm := (abs(width-x)<BORDERMARGIN);
  bm := (abs(height-y)<BORDERMARGIN);

  c := crdefault;

  // simple edge conditions
  if bm then c := crSizeNS;
  if tm then c := crSize;
  if (lm or rm) then c := crSizeWE;
  // corners
  if ((lm and tm) or (rm and bm)) then c := crSizeNWSE;
  if ((rm and tm) or (lm and bm)) then c := crSizeNESW;

  // change if altered
  if cursor <> c then cursor := c;
end;

procedure TfExperiment.MouseDown(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
{ if we're in the border regions as determined by the cursor shape
  then we have to initiate a border resize }
var
  resizeaction : integer;
const
  sc_DragMove = $f012;
  sc_Leftsize = $f001;
  sc_Rightsize = $f002;
  sc_Upsize = $f003;
  sc_UpLeftsize = $f004;
  sc_DnRightsize = $f005;
  sc_Dnsize = $f006;
  sc_DnLeftsize = $f007;
  sc_UpRightsize = $f008;
begin

  inherited MouseDown(Button,Shift,X,Y);

  if cursor = crdefault then exit;  // nothing to do

  // wipe out the shading bitmap to force new shading
  bm.free;
  bm := nil;

  // start the resize logic
  Resizeaction := 0;    // safety trap
  ReleaseCapture;
  if cursor = crSizeWE then begin
    if X < BORDERMARGIN then resizeaction := sc_LeftSize
                        else resizeaction := sc_RightSize;
  end;
  if cursor = crSizeNS then resizeaction := sc_DnSize;
  if cursor = crSizeNWSE then begin
    if X < BORDERMARGIN then resizeaction := sc_UpLeftSize
                        else resizeaction := sc_UpRightSize;
  end;
  if cursor = crSizeNESW then begin
    if X < BORDERMARGIN then resizeaction := sc_DnLeftSize
                        else resizeaction := sc_DnRightSize;
  end;
  if cursor = crSize then resizeaction := sc_DragMove;

  // do the dirty deed
  if resizeaction>0 then Perform(wm_SysCommand, resizeaction, 0);

end;



procedure TfExperiment.SpeedButton1Click(Sender: TObject);
begin
  application.processmessages;
  setup3d;
end;


constructor TfExperiment.Create(AOwner: TComponent);
begin
  inherited create(AOwner);
  SetupSomeStdShades;
  Fwallpaper := wpFilligree;
  currentshadeindex := 0;
  SetDefaultWallpaper(integer(FWallpaper));
  DefaultRaised := true;
  SetRaisedStyle(true,1);
end;


procedure TfExperiment.Resize;
begin
  inherited resize;
  setup3d;
end;

procedure TfExperiment.Activate;
begin
  inherited activate;
  setup3d;
end;

procedure TfExperiment.buBitmapClick(Sender: TObject);
begin
  bitmapnumber := bitmapnumber + 1;
  if bitmapnumber > 22 then bitmapnumber := 1;
  setup3d;
end;

procedure TfExperiment.buShadeClick(Sender: TObject);
begin
  shadenumber := shadenumber + 1;
  if shadenumber > 11 then shadenumber := 0;
  setup3d;
end;

end.
