unit shades;
(**************************************************************************
*****                                                                 *****
*****   SHADES                                                        *****
*****                                                                 *****
*****   A unit for creating effects with transparent forms            *****
*****                                                                 *****
*****   Copyright Peter Fox 2000                                      *****
*****   peter@eminent.demon.co.uk                                     *****
*****                                                                 *****
*****                                                                 *****
*****                                                                 *****
*****                                                                 *****
*****                                                                 *****
**************************************************************************)



interface

  uses windows,graphics,forms,controls,sysutils,extctrls;

  Type
    TShade = String;
    TWallpaper = (wpNone,wpShade,wpTriSteel,wpHessian,
                  wpMarble,wpDicast,wpFilligree,wpLeather,
                  wpWood,wpKaliedescope,wpLightning);


  function Brightness(const clr:Tcolor):integer;
  function CaptureActualScreenRect( ARect: TRect ): TBitmap;
  Function CreateShade(const a : array of integer):TShade;
  function AdjustShade(const clr:Tcolor;const sh:integer):Tcolor;
  Function FormBitmap(aForm:TForm):TBitmap;
  Function BorderShadeForm(aForm:TForm;Shades:Tshade;WithWallpaper:boolean):TBitmap;
  Function PictureShadeForm(aForm:TForm;Shades:Tshade;PictureName:string):TBitmap;
  procedure BorderShade(var bm:Tbitmap; const shades : TShade);
  procedure SetUpSomeStdShades;
  procedure RaiseAndSinkControls(aForm:Tform);
  procedure RaiseControls(aForm:Tform);
  procedure RaiseControl(var bm:Tbitmap; c:tcontrol);
  procedure SetRaisedStyle(const Floating:boolean;const Strength:integer);
  Procedure SetDefaultWallpaper(const wpid:integer);
  procedure SinkControls(aForm:Tform);
  procedure SinkControl(var bm:Tbitmap; c:tcontrol);
  Function  MergeShade(const c1,c2:TColor; const percent:integer):Tcolor;
  Procedure RepaintControls(aForm:Tform);
  Procedure TileBmOntoBm(const BM:Tbitmap; var Dest:TBitmap);

  // for convenience of sharing shades between forms
  // Call SetupSomeShades early on to create a selection of shades
  // CurrentShadeIndex, DefaultWithWallpaper and DefaultRaised
  // are just convenient variables that all callers will be able to access
  // so you can standardise behaviour throughout an application.
  var
     ShadeStore : array [0..19] of Tshade;
     CurrentShadeindex : integer;
     ShadeLimit : integer;                   // highest shade currently in shadestore
     DefaultWithWallpaper : boolean;
     DefaultWallpaper     : string;
     DefaultRaised   : boolean;
     DefaultFloating : boolean;
     DefaultStrength : integer;
     RaisedStyleSet  : boolean;
     raisedshade : array [1..4] of array [0..5] of integer;


implementation

{$R shades.res}


function Brightness(const clr:Tcolor):integer;
{ return the average intensity of the RGB values in the colour provided
  255 = white 0=black }
var
  c : integer;
  r,g,b : integer;
begin
  c := clr;
  r := c and $FF;
  c := c shr 8;
  g := c and $FF;
  c := c shr 8;
  b := c and $FF;
  result := (r+g+b) div 3;
end;


procedure SetRaisedStyle(const Floating:boolean;const Strength:integer);
{ set the styles to used for raised controls }
var
  str,i,sh : integer;
begin
  defaultFloating := Floating;
  Defaultstrength := Strength;
  if defaultstrength < 1 then defaultstrength := 1;
  if defaultstrength > 4 then defaultstrength := 4;
  for i := 0 to 5 do begin
    sh := 70+(i*5);
    for str := 1 to 4 do begin
      if str = 1 then raisedshade[str,i] := sh
                 else raisedshade[str,i] := (raisedshade[str-1,i] * sh) div 100;
    end;
  end;
  RaisedStyleSet := true;
end;

procedure SetUpSomeStdShades;
{ put some generic shades into the shade store
  0 ... Raised quarter rounded
  1 ... Sunk quarter rounded
  2 ... Picture frame moulding
  3 ... Sunk Quarter round (larger than 1 with defining top edge)
  4 ... Small half round moulding
  5 ... deep moulding
  6 ... simple raised
  7 ... simple sunk
  8 ... rail
  9 ... plateau
  10 .. incised half rounds
  11 .. large half round raised

}
begin
  // raised rounded
  ShadeStore[0] := CreateShade([50,56,59,63,67,71,75,79,84,88,90,93,96,99,102,105,108]);
  ShadeStore[1] := CreateShade(  [99,98,97,96,94,92,89,85,82,78,74,69,40,
     130,125,120,110,109,108,107,106,105,104,103,102,101]);
  ShadeStore[2] := CreateShade([70,150,84,79,86,100,113,117,111,99,89,86,92]);
  ShadeStore[3] := CreateShade([70,108,105,102,99,96,93,90,88,84,79,75,71,67,63,59,56,50]);
  ShadeStore[4] := CreateShade([60,80,85,90,120,120,90,85,80,60]);
  ShadeStore[5] := CreateShade([90,100,130,70,60,50,60,70,130,100,90]);
  ShadeStore[6] := CreateShade([60,100,140,150,140,100,60]);
  ShadeStore[7] := CreateShade([140,60,50,60,140]);
  ShadeStore[8] := CreateShade([50,80,80,60,60,110,110,80,80,50]);
  shadestore[9] := createshade([99,97,94,90,85,79,72,64,53]);
  shadestore[10] := createshade([99,97,94,90,85,79,72,64,53,40,53,64,72,79,85,90,94,97,99]);
  shadestore[11] := createshade([53,64,72,79,85,90,94,97,99,97,94,90,85,79,72,64,53]);
  shadelimit := 11;


end;


procedure SinkControls(aForm:Tform);
{ draw a highlight along the right and bottom edges
  draw a shadow along the top and left edges
}
var
  ci : integer;
  bm : tbitmap;
  C : Tcontrol;
begin
  // trap forgetting to explicitly specify style
  if not raisedstyleset then setraisedstyle(true,1);

  bm := formbitmap(aform);
  TRY  // protect BM resource

  for ci := 0 to aform.controlcount-1 do begin
    C := aform.controls[ci];
    if not c.visible then continue;
    sinkcontrol(bm,C)
  end;
  aForm.canvas.draw(0,0,bm);
  FINALLY
    bm.free;
  END;

end;


procedure RaiseAndSinkControls(aForm:Tform);
{  Raise images (buttons) but sink the rest
   See RaiseControls and SinkControls
}
var
  ci : integer;
  bm : tbitmap;
  C : Tcontrol;
begin
  // trap forgetting to explicitly specify style
  if not raisedstyleset then setraisedstyle(true,2);

  bm := formbitmap(aform);
  TRY  // protect BM resource

  for ci := 0 to aform.controlcount-1 do begin
    C := aform.controls[ci];
    if not c.visible then continue;
    if c is timage then raisecontrol(bm,C)
                   else sinkcontrol(bm,C);
  end;
  aForm.canvas.draw(0,0,bm);
  FINALLY
    bm.free;
  END;

end;



procedure RaiseControls(aForm:Tform);
{ draw a shadow along the right and bottom edges
  If floating is true then give the impression that the control is
  poised above the form with invisible support.  If Floating is false then
  show the control on a solid plinth
}
var
  ci : integer;
  bm : tbitmap;
  C : Tcontrol;
begin
  // trap forgetting to explicitly specify style
  if not raisedstyleset then setraisedstyle(true,2);

  bm := formbitmap(aform);
  TRY  // protect BM resource

  for ci := 0 to aform.controlcount-1 do begin
    C := aform.controls[ci];
    if not c.visible then continue;
    raisecontrol(bm,C);
  end;
  aForm.canvas.draw(0,0,bm);
  FINALLY
    bm.free;
  END;

end;


procedure SinkControl(var bm:Tbitmap; c:tcontrol);
{ sink a single control }
var
  i,x,y,sh : integer;
  RE,BE,LE,TE :integer;
  fromlim,tolim:integer;
begin
  LE := C.left;
  TE := C.top;
  RE := LE+C.width;
  BE := TE+C.height;

  for i := 0 to 5 do begin
    sh := raisedshade[defaultstrength,i];
    fromlim := i;
    tolim := i;
    //if defaultfloating then fromlim := 5 else fromlim := i;
    //if defaultfloating then tolim := i-1 else tolim := i;
    // lh edge shadow
    x := LE -i;
    for y := TE-fromlim to BE-tolim do begin
      bm.canvas.pixels[x,y]:=adjustshade(bm.canvas.pixels[x,y],sh);
    end;
    // top edge shadow
    y := TE -i;
    for x := LE to RE-i do begin
      bm.canvas.pixels[x,y]:=adjustshade(bm.canvas.pixels[x,y],sh);
    end;
    sh := 100-(2*(sh-100));  // convert shade to highlight
    // rh edge highlight
    x := RE +i;
    for y := TE-fromlim to BE+tolim do begin
      bm.canvas.pixels[x,y]:=adjustshade(bm.canvas.pixels[x,y],sh);
    end;
    // bottom edge highlight
    y := BE +i;
    for x := LE to RE+i do begin
      bm.canvas.pixels[x,y]:=adjustshade(bm.canvas.pixels[x,y],sh);
    end;
  end;
end;


procedure RaiseControl(var bm:Tbitmap; c:tcontrol);
{ raise a single control }
var
  i,x,y,sh : integer;
  RE,BE,LE,TE :integer;
  fromlim,tolim:integer;
begin
  LE := C.left;
  TE := C.top;
  RE := LE+C.width;
  BE := TE+C.height;

  for i := 0 to 5 do begin
    sh := raisedshade[defaultstrength,i];
    if defaultfloating then fromlim := 5 else fromlim := i;
    if defaultfloating then tolim := i-1 else tolim := i;
    // rh edge
    x := RE +i;
    for y := TE+fromlim to BE+tolim do begin
      bm.canvas.pixels[x,y]:=adjustshade(bm.canvas.pixels[x,y],sh);
    end;
    // bottom edge
    y := BE +i;
    for x := LE+fromlim to RE+i do begin
      bm.canvas.pixels[x,y]:=adjustshade(bm.canvas.pixels[x,y],sh);
    end;
  end;
end;



procedure BorderShade(var bm:Tbitmap; const shades : TShade);
{ apply a set of shades to the edges of the supplied bitmap }
var
  sh,x,y : integer;
begin
  with bm do begin
    // left and right edges
    for x := 0 to length(shades)-1  do begin
      for y := x to (height-1)-x do begin
        sh := ord(shades[x+1]);
        canvas.pixels[x,y]:=adjustshade(canvas.pixels[x,y],sh);
        canvas.pixels[width-x-1,y]:=adjustshade(canvas.pixels[width-x-1,y],sh);
      end;
      application.processmessages;
    end;
    // top and bottom edges
    for y := 0 to length(shades)-1 do begin
      sh := ord(shades[y+1]);
      for x := y to (width -1)-y do begin
        canvas.pixels[x,y]:=adjustshade(canvas.pixels[x,y],sh);
        canvas.pixels[x,height-y-1]:=adjustshade(canvas.pixels[x,height-y-1],sh);
      end;
      application.processmessages;
    end;
  end;
end;


Function MergeShade(const c1,c2:TColor; const percent:integer):Tcolor;
{ returns a colour somewhere between c1 and c2 (each bit being adjusted in proportion)
  0 percent returns c1 ... 100 percent returns c2 }
var
  rgb1 : array [0..2] of integer;
  rgb2 : array [0..2] of integer;
  rgb3 : array [0..2] of integer;
  i,c : integer;
begin
  // split the colours into components
  c := c1;
  rgb1[0] := c and $FF;c := c shr 8;
  rgb1[1] := c and $FF;c := c shr 8;
  rgb1[2] := c and $FF;
  c := c2;
  rgb2[0] := c and $FF;c := c shr 8;
  rgb2[1] := c and $FF;c := c shr 8;
  rgb2[2] := c and $FF;
  // adjust in proportion
  for i := 0 to 2 do rgb1[i] := rgb1[i] * (100-percent);
  for i := 0 to 2 do rgb2[i] := rgb2[i] * percent;
  // merge
  for i := 0 to 2 do rgb3[i] := (rgb1[i] + rgb2[i]) div 100;
  // construct result
  result := (rgb3[2] shl 16) + (rgb3[1] shl 8) + rgb3[0];
end;


function AdjustShade(const clr:Tcolor;const sh:integer):Tcolor;
{ darkens or lightens the supplied colour.
  sh < 100 ... Darken
  sh > 100 ... Lighten }
var
  lighten : boolean;
  i,c,k,s : integer;
  rgb : array [0..2] of integer;
begin
  // trap 100% = do nothing
  if sh = 100 then begin
    result := clr;
    exit;
  end;

  lighten := (sh > 100);

  // split colour into three components
  c := clr;
  rgb[0] := c and $FF;c := c shr 8;
  rgb[1] := c and $FF;c := c shr 8;
  rgb[2] := c and $FF;


  // move each component towards 0 or 255 by the specified proportion
  for i := 0 to 2 do begin
    if not lighten then begin
      rgb[i] := (rgb[i]*sh) div 100;
    end else begin
      k := 255-rgb[i];
      s := 200-sh;
      rgb[i] := 255-((k*s) div 100);
    end;
  end;

  result := (rgb[2] shl 16) + (rgb[1] shl 8) + rgb[0];
end;


Function FormBitmap(aForm:TForm):TBitmap;
{ Returns a bitmap copied from the form's canvas }
begin
  result := TBitmap.Create;
  with result, aForm.Boundsrect do begin
    Width := Right - Left;
    Height := Bottom - Top;
    BitBlt( Canvas.Handle, 0, 0, Width, Height,
      aForm.canvas.handle, 0,0, SRCCOPY);
  end;
end;


Procedure RepaintControls(aForm:Tform);
{ after whacking bitmaps onto the form with BorderShadeForm and PictureShadeForm
  the images get wiped out.  This repaints them (if any) }
var
  ci : integer;
  c  : Tcontrol;
begin
  for ci := 0 to aform.controlcount-1 do begin
    C := aform.controls[ci];
    if not c.visible then continue;
    c.repaint;
  end;
end;


Function BorderShadeForm(aForm:TForm;Shades:Tshade;WithWallpaper:boolean):TBitmap;
{ apply shading to the form border.
  If WithWallpaper is true then fetch the underlying desktop wallpaper first.

  The return value is the shaded bitmap which might be needed for repainting.
  To repaint you wold use   MyForm.canvas.draw(0,0,ShadedBitmap);}
begin
  if WithWallpaper then begin             // Possibly...
    paintdesktop(aform.canvas.handle);    // get the underlying wallpaper into the form
    application.processmessages;
  end;
  result := FormBitmap(aForm);            // retrieve the bitmap from the form's canvas
  BorderShade(result,shades);             // shade the border of the bitmap
  aForm.canvas.draw(0,0,result);          // paint the form canvas with shaded bitmap
  RepaintControls(aForm);
end;



Function PictureShadeForm(aForm:TForm;Shades:Tshade;PictureName:string):TBitmap;
{ tile a picture onto the form then shade.
  If Picturename is null string do no background painting
  If Picturename is "WALLPAPER" then use the desktop wallpaper
  If Picturename is "DEFAULT" then use whatever has been set up in the default with SetDefaultWallpaper()
  If Picturename is anything else then this is taken to be the
    name of a bitmap *resource* specified in the shades.rc and
    compiled into shades.res  eg 'BG007'
      Current contents of shades.rc
      -----------------------------------
      \    BG001  BITMAP line7.bmp
      \    BG002  BITMAP tristeel.bmp
      \    BG003  BITMAP hess.bmp
      \    BG004  BITMAP rosemarble.bmp
      \    BG005  BITMAP chalktry.bmp
      \    BG006  BITMAP goldtry.bmp
      \    BG007  BITMAP stonetry.bmp
      \    BG008  BITMAP woodtry.bmp
      \    BG009  BITMAP kalied5a.bmp
      \    BG010  BITMAP lightng.bmp


  The return value is the shaded bitmap which might be needed for repainting.
  To repaint you would use   MyForm.canvas.draw(0,0,ShadedBitmap);}
var
  tbm : Tbitmap;
  pname : string;
begin
  tbm := nil;
  pname := PictureName;
  if pname = 'DEFAULT' then pname := DefaultWallpaper;

  If pname <> '' then begin         // some graphic overlay first
    if pname = 'WALLPAPER' then begin
      paintdesktop(aform.canvas.handle);    // get the underlying wallpaper into the form
    end else begin
      tbm := tBitmap.create;
      try
        tbm.LoadFromResourceName(HInstance,PName);
      except
        tbm.free;
        tbm := nil;
      end;
    end;
    application.processmessages;
  end;

  result := FormBitmap(aForm);            // retrieve the bitmap from the form's canvas

  if assigned(tbm) then begin             // tile picture onto it
    TileBmOntoBm(tbm,result);
    tbm.free;
  end;

  BorderShade(result,shades);             // shade the border of the bitmap
  aForm.canvas.draw(0,0,result);          // paint the form canvas with shaded bitmap
  RepaintControls(aForm);
end;

Procedure TileBmOntoBm(const BM:Tbitmap; var Dest:TBitmap);
{ tile the BM bitmap onto the DEST bitmap }
var
  x,y:integer;
begin
  x := 0;
  while x <= dest.width do begin
    y := 0;
    while y <= dest.height do begin
      dest.canvas.draw(x,y,BM);
      y := y + BM.height;
    end;
    x := x + BM.width;
  end;
end;




Function CreateShade(const a : array of integer):TShade;
var
  i : integer;
begin
  result := '';
  for i := 0 to high(a) do result := result+chr(a[i]);
end;

function CaptureActualScreenRect( ARect: TRect ): TBitmap;
{ use this to capture a rectangle on the screen
  This is the actual current display not the wallpaper/background}
var
  ScreenDC: HDC;
begin
  result := TBitmap.Create;
  with Result, ARect do
  begin
    Width := Right - Left;
    Height := Bottom - Top;

    ScreenDC := GetDC( 0 );
    try
    	BitBlt( Canvas.Handle, 0, 0, Width, Height,
      	ScreenDC, Left, Top, SRCCOPY);
    finally
    	ReleaseDC( 0, ScreenDC );
    end;
  end;
end;

Procedure SetDefaultWallpaper(const wpid:integer);
{ use an integer to generate the string that we pass to pictureshadeform}
begin
  case wpid of
  -1 :  DefaultWallpaper := 'WALLPAPER';
   0 :  DefaultWallpaper := '';
  else
    DefaultWallpaper := 'BG' + format('%3.3d',[wpid]);
  end;
end;


end.
